/*
Copyright (C) 2006 Evan Teran
                   eteran@alum.rit.edu

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "ByteShiftArray.h"
#include <algorithm>

//------------------------------------------------------------------------------
// Name: ByteShiftArray(std::size_t size)
// Desc: constructor
//------------------------------------------------------------------------------
ByteShiftArray::ByteShiftArray(std::size_t size) : m_Size(size) {
	m_Data = new uint8_t[m_Size];
	clear();
}

//------------------------------------------------------------------------------
// Name: ByteShiftArray(const ByteShiftArray& other)
// Desc: copy constructor
//------------------------------------------------------------------------------
ByteShiftArray::ByteShiftArray(const ByteShiftArray& other) {
	if(&other != this) {
		m_Size = other.m_Size;
		m_Data = new uint8_t[m_Size];
		std::copy(m_Data, m_Data + m_Size, other.m_Data);
	}
}

//------------------------------------------------------------------------------
// Name: ~ByteShiftArray()
// Desc: destructor
//------------------------------------------------------------------------------
ByteShiftArray::~ByteShiftArray() {
	delete [] m_Data;
}

//------------------------------------------------------------------------------
// Name: shl()
// Desc: shifts data left one byte and shifts in a 0
//------------------------------------------------------------------------------
ByteShiftArray &ByteShiftArray::shl() {

	for(std::size_t i = 0; i < (m_Size - 1); ++i) {
		m_Data[i] = m_Data[i + 1];
	}
	
	m_Data[m_Size - 1] = 0;

	return *this;
}

//------------------------------------------------------------------------------
// Name: shr()
// Desc: shifts data right one byte and shifts in a 0
//------------------------------------------------------------------------------
ByteShiftArray &ByteShiftArray::shr() {
	for(std::size_t i = m_Size - 1; i >= 1; --i) {
		m_Data[i] = m_Data[i - 1];
	}
	
	m_Data[0] = 0;
	return *this;
}

//------------------------------------------------------------------------------
// Name: size() const
// Desc: returns size of this byte array
//------------------------------------------------------------------------------
std::size_t ByteShiftArray::size() const {
	return m_Size;
}

//------------------------------------------------------------------------------
// Name: operator[](std::size_t i)
// Desc: returns and l-value version of an element in the byte array
//------------------------------------------------------------------------------
uint8_t &ByteShiftArray::operator[](std::size_t i) {
	return m_Data[i];
}

//------------------------------------------------------------------------------
// Name: operator[](std::size_t i) const
// Desc: returns and r-value version of an element in the byte array
//------------------------------------------------------------------------------
const uint8_t ByteShiftArray::operator[](std::size_t i) const {
	return m_Data[i];
}

//------------------------------------------------------------------------------
// Name: data() const
// Desc: returns a read only pointer to the data this byte array holds
//------------------------------------------------------------------------------
const uint8_t *ByteShiftArray::data() const {
	return m_Data;
}

//------------------------------------------------------------------------------
// Name: clear()
// Desc: zeroes out the byte array
//------------------------------------------------------------------------------
void ByteShiftArray::clear() {
	std::fill(m_Data, m_Data + m_Size, 0);
}
